const { expect } = require('chai');
const async = require('async');
const http = require('http');
const fetch = require('node-fetch');
const createServer = require('../example/sample');

describe('Proxy Routes', () => {
  const proxyServerPort = 6010;
  const targetPort = 8080;
  const targetFQDN = '127.0.0.1';

  before((done) => {
    // runs before all tests in this block
    http.createServer((req, res) => {
      res.writeHead(200, { 'Content-Type': 'application/json' });
      return res.end(JSON.stringify({
        // response includes the url that you tried to access
        translatedPath: req.url,
      }));
    }).listen(targetPort, () => {
      createServer(proxyServerPort, () => {
        done(); // call done to start running test suite
      });
    });
  });

  it('should translate the url correctly', (done) => {
    const urlTranslationTests = [{
      // visited path matches rule (a.1).
      visitedPath: '/a/test',
      newUrlTarget: `${targetFQDN}:${targetPort}/cool`,
    }, {
      // visited path matches rule (a.1).
      // whatever portion is not part of the match is carried over
      // to the new path.
      // in this case, the '/' is carried over.
      visitedPath: '/a/test/',
      newUrlTarget: `${targetFQDN}:${targetPort}/cool/`,
    }, {
      // visited path matches rule (1).
      // query parameters are carried over.
      visitedPath: '/a/test?hi=5/',
      newUrlTarget: `${targetFQDN}:${targetPort}/cool?hi=5/`,
    }, {
      // visited path matches rule (2).
      // the unmatched portion '/yo' is carried over.
      visitedPath: '/a/test2/yo',
      newUrlTarget: `${targetFQDN}:${targetPort}/cool2/yo`,
    }, {
      // visited path matches rule (1).
      // note that the key is interpreted as a regex expression and the
      // module matches against the visited path, and not the entire url.
      visitedPath: '/a/fuzzyshoe/test',
      newUrlTarget: `${targetFQDN}:${targetPort}/cool`,
    }, {
      // visited path matches rule (1).
      // the unmatched portion '/seven' is carried over.
      visitedPath: '/a/test/seven',
      newUrlTarget: `${targetFQDN}:${targetPort}/cool/seven`,
    }, {
      // no rule matched, so the module uses the specified default target.
      // the entire visited path is carried over.
      // see the `Other Notes` section of README to see why specifying
      // the rule `.*/test` does not match `/testalmost`.
      visitedPath: '/a/testalmost',
      newUrlTarget: `${targetFQDN}:${targetPort}/testalmost`,
    }, {
      // no rule matched, so the module uses the specified default target.
      // the entire visited path is carried over.
      visitedPath: '/a/testalmost/5',
      newUrlTarget: `${targetFQDN}:${targetPort}/testalmost/5`,
    }, {
      // visited path matched rule (3).
      // regex placeholder should be matched and replaced correctly
      visitedPath: '/a/posts/11/comments/12',
      newUrlTarget: `${targetFQDN}:${targetPort}/p/11/c/12`,
    }, {
      // visited path matches rule (3).
      // whatever portion is not part of the match is carried over
      // to the new path.
      // in this case, the '/' is carried over.
      visitedPath: '/a/posts/11/comments/12/',
      newUrlTarget: `${targetFQDN}:${targetPort}/p/11/c/12/`,
    }, {
      // visited path matches rule (3).
      // query parameters are carried over.
      visitedPath: '/a/posts/11/comments/12?hi=5/',
      newUrlTarget: `${targetFQDN}:${targetPort}/p/11/c/12?hi=5/`,
    }, {
      // visited path matches rule (4).
      // parameters substitution should work correctly for rules with slash at the end
      visitedPath: '/a/author/11/posts/12/',
      newUrlTarget: `${targetFQDN}:${targetPort}/a/11/p/12/`,
    }, {
      // visited path matches a rule default.
      // 没有匹配的规则,使用a rule default会重写到a rule (3)
      visitedPath: '/a/default/a?a=123',
      newUrlTarget: `${targetFQDN}:${targetPort}/default/a?a=123`,
    }, {
      // visited path matches b rule default.
      // 没有匹配的规则,使用default会重写到a rule (3)
      visitedPath: '/b/posts/11/comments/12?hi=5/',
      newUrlTarget: `${targetFQDN}:${targetPort}/p/11/c/12?hi=5/`,
    }, {
      // visited path matches b rule (2).
      // 会重写到a rule (3)
      visitedPath: '/b/p/11/c/12',
      newUrlTarget: `${targetFQDN}:${targetPort}/p/11/c/12`,
    },
    ];

    // makes a bunch of requests in parallel, and then calls
    // `done` after we receive all responses back
    async.each(urlTranslationTests, async (comparisonObj) => {
      const res = await fetch(`http://127.0.0.1:${proxyServerPort}${comparisonObj.visitedPath}`);
      expect(res.status).to.equal(200);
      const json = await res.json();
      expect(`${targetFQDN}:${targetPort}${json.translatedPath}`).to.equal(comparisonObj.newUrlTarget);
    }, done);
  });

  it('loop rewrite', async () => {
    const res = await fetch(`http://127.0.0.1:${proxyServerPort}/a/loop/rewrite`);
    expect(res.status).to.equal(500);
    expect(await res.text()).to.equal('rewrite 超过10次');
  });
});
